from ninja_extra.interfaces.ordering import OrderingBase

from .decorator import ordering
from .models import Ordering
from .operation import AsyncOrderatorOperation, OrderatorOperation

__all__ = [
    "OrderingBase",
    "Ordering",
    "ordering",
    "OrderatorOperation",
    "AsyncOrderatorOperation",
]



from ninja_extra.interfaces.searching import SearchingBase

from .decorators import searching
from .models import Searching
from .operations import AsyncSearcheratorOperation, SearcheratorOperation

__all__ = [
    "SearchingBase",
    "Searching",
    "searching",
    "SearcheratorOperation",
    "AsyncSearcheratorOperation",
]



import inspect
from typing import Any, Callable, List, Type, Union, cast

from ninja.throttling import BaseThrottle

from ninja_extra.constants import THROTTLED_FUNCTION, THROTTLED_OBJECTS
from ninja_extra.lazy import settings_lazy


def throttle(
    *func_or_throttle_klass_or_object: Any, **init_kwargs: Any
) -> Callable[..., Any]:
    isfunction = (
        inspect.isfunction(func_or_throttle_klass_or_object[0])
        if len(func_or_throttle_klass_or_object) == 1
        else False
    )

    if isfunction:
        func = func_or_throttle_klass_or_object[0]
        throttle_classes: List[Type[BaseThrottle]] = cast(
            List[Type[BaseThrottle]], settings_lazy().THROTTLE_CLASSES
        )
        return _inject_throttling(func, *throttle_classes, **init_kwargs)

    def wrapper(view_func: Callable[..., Any]) -> Any:
        return _inject_throttling(
            view_func, *func_or_throttle_klass_or_object, **init_kwargs
        )

    return wrapper


def _lazy_throttling_objects(
    *throttle_klass_or_object: Union[Type[BaseThrottle], BaseThrottle],
    **init_kwargs: Any,
) -> Callable:
    def _() -> List[BaseThrottle]:
        res = []
        for item in throttle_klass_or_object:
            if isinstance(item, type):
                res.append(item(**init_kwargs))
                continue

            res.append(item)

        return res

    return _


def _inject_throttling(
    func: Callable[..., Any],
    *throttle_classes: Type[BaseThrottle],
    **init_kwargs: Any,
) -> Callable[..., Any]:
    throttling_objects_lazy: Callable = _lazy_throttling_objects(
        *throttle_classes, **init_kwargs
    )
    setattr(func, THROTTLED_FUNCTION, True)
    setattr(func, THROTTLED_OBJECTS, throttling_objects_lazy)

    return func



import operator
import re
import typing as t
from functools import reduce

from django.db.models import Q, QuerySet
from django.db.models.constants import LOOKUP_SEP
from ninja import Field, Schema

from ninja_extra.interfaces.searching import SearchingBase


def _istartswith(a: str, b: str) -> bool:
    return a.startswith(b)


def _isiexact(a: str, b: str) -> bool:
    return a.lower() == b.lower()


def _isiregex(a: str, b: str) -> bool:
    return bool(re.search(b, a, re.IGNORECASE))


def _isicontains(a: str, b: str) -> bool:
    return b.lower() in a.lower()


class Searching(SearchingBase):
    class Input(Schema):
        search: t.Optional[str] = Field(None)

    lookup_prefixes = {
        "^": "istartswith",
        "=": "iexact",
        "@": "search",
        "$": "iregex",
    }

    lookup_prefixes_list = {
        "^": _istartswith,
        "=": _isiexact,
        "$": _isiregex,
    }

    def __init__(
        self,
        search_fields: t.Optional[t.List[str]] = None,
        pass_parameter: t.Optional[str] = None,
    ) -> None:
        super().__init__(pass_parameter=pass_parameter)
        self.search_fields = search_fields or []

    def searching_queryset(
        self, items: t.Union[QuerySet, t.List], searching_input: Input
    ) -> t.Union[QuerySet, t.List]:
        search_terms = self.get_search_terms(searching_input.search)

        if self.search_fields and search_terms:
            if isinstance(items, QuerySet):
                conditions_queryset = self.construct_conditions_for_queryset(
                    search_terms
                )
                return items.filter(reduce(operator.and_, conditions_queryset))
            elif isinstance(items, list):
                conditions_list = self.construct_conditions_for_list(search_terms)
                return [
                    item for item in items if self.filter_spec(item, conditions_list)
                ]
        return items

    def get_search_terms(self, value: t.Optional[str]) -> t.List[str]:
        if value:
            value = value.replace("\x00", "")  # strip null characters
            value = value.replace(",", " ")
            return value.split()
        return []

    def construct_search(self, field_name: str) -> str:
        lookup = self.lookup_prefixes.get(field_name[0])
        if lookup:
            field_name = field_name[1:]
        else:
            lookup = "icontains"
        return LOOKUP_SEP.join([field_name, lookup])

    def construct_conditions_for_queryset(self, search_terms: t.List[str]) -> t.List[Q]:
        orm_lookups = [
            self.construct_search(str(search_field))
            for search_field in self.search_fields
        ]
        conditions = []
        for search_term in search_terms:
            queries = [Q(**{orm_lookup: search_term}) for orm_lookup in orm_lookups]
            conditions.append(reduce(operator.or_, queries))
        return conditions

    def construct_conditions_for_list(
        self, search_terms: t.List[str]
    ) -> t.Dict[str, t.List[t.Tuple[t.Callable, str]]]:
        lookups = self.construct_search_for_list()
        conditions: t.Dict[str, t.List[t.Tuple[t.Callable, str]]] = {
            field_name: [] for field_name in lookups.keys()
        }
        for search_term in search_terms:
            for field_name, lookup in lookups.items():
                conditions[field_name].append((lookup, search_term))
        return conditions

    def construct_search_for_list(self) -> t.Dict[str, t.Callable]:
        def get_lookup(prefix: str) -> t.Callable:
            return self.lookup_prefixes_list.get(prefix, _isicontains)

        return {
            field_name[1:]
            if (self.lookup_prefixes_list.get(field_name[0]))
            else field_name: get_lookup(field_name[0])
            for field_name in self.search_fields
        }

    def filter_spec(
        self, item: t.Any, conditions: t.Dict[str, t.List[t.Tuple[t.Callable, str]]]
    ) -> bool:
        item_getter = (
            operator.itemgetter if isinstance(item, dict) else operator.attrgetter
        )
        for field, lookup in conditions.items():
            if not any(
                lookup_func(item_getter(field)(item), lookup_value)
                for lookup_func, lookup_value in lookup
            ):
                return False
        return True



import typing as t

import pydantic
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse
from django.http.request import HttpRequest
from ninja.errors import ValidationError
from ninja.types import DictStrAny

from ninja_extra.interfaces.route_context import RouteContextBase
from ninja_extra.lazy import settings_lazy
from ninja_extra.types import PermissionType

if t.TYPE_CHECKING:
    from ninja_extra.details import ViewSignature
    from ninja_extra.main import NinjaExtraAPI


class RouteContext(RouteContextBase):
    """
    APIController Context which will be available to the class instance when handling request
    """

    __slots__ = [
        "permission_classes",
        "request",
        "response",
        "args",
        "kwargs",
        "_api",
        "_view_signature",
        "_has_computed_route_parameters",
    ]

    permission_classes: PermissionType
    request: t.Union[t.Any, HttpRequest, None]
    response: t.Union[t.Any, HttpResponse, None]
    args: t.List[t.Any]
    kwargs: DictStrAny

    def __init__(
        self,
        request: HttpRequest,
        args: t.Optional[t.List[t.Any]] = None,
        permission_classes: t.Optional[PermissionType] = None,
        kwargs: t.Optional[DictStrAny] = None,
        response: t.Optional[HttpResponse] = None,
        api: t.Optional["NinjaExtraAPI"] = None,
        view_signature: t.Optional["ViewSignature"] = None,
    ):
        self.request = request
        self.response = response
        self.args: t.List[t.Any] = args or []
        self.kwargs: DictStrAny = kwargs or {}
        self.kwargs.update({"view_func_kwargs": {}})
        self.permission_classes: PermissionType = permission_classes or []
        self._api = api
        self._view_signature = view_signature
        self._has_computed_route_parameters = False

    @property
    def has_computed_route_parameters(self) -> bool:
        return self._has_computed_route_parameters

    def compute_route_parameters(
        self,
    ) -> None:
        if self._view_signature is None or self._api is None:
            raise ImproperlyConfigured(
                "view_signature and api are required. "
                "Or you are taking an approach that is not supported "
                "RouteContext to compute route parameters."
            )

        if self._has_computed_route_parameters:
            return

        values, errors = {}, []
        for model in self._view_signature.models:
            try:
                data = model.resolve(self.request, self._api, self.kwargs)
                values.update(data)
            except pydantic.ValidationError as e:
                items = []
                for i in e.errors(include_url=False):
                    i["loc"] = (
                        model.__ninja_param_source__,
                    ) + model.__ninja_flatten_map_reverse__.get(i["loc"], i["loc"])
                    # removing pydantic hints
                    del i["input"]  # type: ignore
                    if (
                        "ctx" in i
                        and "error" in i["ctx"]
                        and isinstance(i["ctx"]["error"], Exception)
                    ):
                        i["ctx"]["error"] = str(i["ctx"]["error"])
                    items.append(dict(i))
                errors.extend(items)

        if errors:
            raise ValidationError(errors)

        if self._view_signature.response_arg:
            values[self._view_signature.response_arg] = self.response

        self.kwargs.update({"view_func_kwargs": values}, **values)
        self._has_computed_route_parameters = True


def get_route_execution_context(
    request: HttpRequest,
    temporal_response: t.Optional[HttpResponse] = None,
    permission_classes: t.Optional[PermissionType] = None,
    api: t.Optional["NinjaExtraAPI"] = None,
    view_signature: t.Optional["ViewSignature"] = None,
    *args: t.Any,
    **kwargs: t.Any,
) -> RouteContext:
    init_kwargs = {
        "permission_classes": permission_classes
        if permission_classes is not None
        else [],
        "request": request,
        "kwargs": kwargs,
        "response": temporal_response,
        "args": args,
        "api": api,
        "view_signature": view_signature,
    }
    context_class = t.cast(
        t.Type[RouteContext],
        settings_lazy().ROUTE_CONTEXT_CLASS,
    )
    context = context_class(**init_kwargs)  # type:ignore[arg-type]
    return context



from abc import ABC, abstractmethod


class RouteContextBase(ABC):
    @property
    @abstractmethod
    def has_computed_route_parameters(self) -> bool: ...

    @abstractmethod
    def compute_route_parameters(self) -> None: ...



import typing as t
from abc import ABC, abstractmethod

from django.db.models import QuerySet
from ninja import Query, Schema


class OrderingBase(ABC):
    class Input(Schema): ...

    InputSource = Query(...)

    def __init__(
        self, *, pass_parameter: t.Optional[str] = None, **kwargs: t.Any
    ) -> None:
        self.pass_parameter = pass_parameter

    @abstractmethod
    def ordering_queryset(
        self, items: t.Union[QuerySet, t.List], ordering_input: t.Any
    ) -> t.Union[QuerySet, t.List]: ...



import logging
import typing as t
from operator import attrgetter, itemgetter

from django.db.models import QuerySet
from ninja import Field, P, Query, Schema
from pydantic import BaseModel

from ninja_extra.interfaces.ordering import OrderingBase

logger = logging.getLogger()


class Ordering(OrderingBase):
    class Input(Schema):
        ordering: t.Optional[str] = Field(None)

    def __init__(
        self,
        ordering_fields: t.Optional[t.List[str]] = None,
        pass_parameter: t.Optional[str] = None,
    ) -> None:
        super().__init__(pass_parameter=pass_parameter)
        self.ordering_fields = ordering_fields or "__all__"
        self.Input = self.create_input(ordering_fields)  # type:ignore

    def create_input(self, ordering_fields: t.Optional[t.List[str]]) -> t.Type[Input]:
        if ordering_fields:

            class DynamicInput(Ordering.Input):
                ordering: Query[t.Optional[str], P(default=",".join(ordering_fields))]  # type:ignore[type-arg,valid-type]

            return DynamicInput
        return Ordering.Input

    def ordering_queryset(
        self, items: t.Union[QuerySet, t.List], ordering_input: Input
    ) -> t.Union[QuerySet, t.List]:
        ordering_ = self.get_ordering(items, ordering_input.ordering)
        if ordering_:
            if isinstance(items, QuerySet):
                return items.order_by(*ordering_)
            elif isinstance(items, list) and items:

                def multisort(xs: t.List, specs: t.List[t.Tuple[str, bool]]) -> t.List:
                    orerator = itemgetter if isinstance(xs[0], dict) else attrgetter
                    for key, reverse in reversed(specs):
                        xs.sort(key=orerator(key), reverse=reverse)
                    return xs

                return multisort(
                    items,
                    [
                        (o[int(o.startswith("-")) :], o.startswith("-"))
                        for o in ordering_
                    ],
                )
        return items

    def get_ordering(
        self, items: t.Union[QuerySet, t.List], value: t.Optional[str]
    ) -> t.List[str]:
        if value:
            fields = [param.strip() for param in value.split(",")]
            return self.remove_invalid_fields(items, fields)
        return []

    def remove_invalid_fields(
        self, items: t.Union[QuerySet, t.List], fields: t.List[str]
    ) -> t.List[str]:
        valid_fields = list(self.get_valid_fields(items))

        def term_valid(term: str) -> bool:
            if term.startswith("-"):
                term = term[1:]
            return term in valid_fields

        return [term for term in fields if term_valid(term)]

    def get_valid_fields(self, items: t.Union[QuerySet, t.List]) -> t.List[str]:
        valid_fields: t.List[str] = []
        if self.ordering_fields == "__all__":
            if isinstance(items, QuerySet):
                valid_fields = self.get_all_valid_fields_from_queryset(items)
            elif isinstance(items, list):
                valid_fields = self.get_all_valid_fields_from_list(items)
        else:
            valid_fields = list(self.ordering_fields)
        return valid_fields

    def get_all_valid_fields_from_queryset(self, items: QuerySet) -> t.List[str]:
        return [str(field.name) for field in items.model._meta.fields] + [
            str(key) for key in items.query.annotations
        ]

    def get_all_valid_fields_from_list(self, items: t.List) -> t.List[str]:
        if not items:
            return []
        item = items[0]
        if isinstance(item, BaseModel):
            return list(item.model_fields.keys())
        if isinstance(item, dict):
            return list(item.keys())
        if hasattr(item, "_meta") and hasattr(item._meta, "fields"):
            return [str(field.name) for field in item._meta.fields]
        return []



import typing as t
import warnings

from .base import ControllerBase, ModelControllerBase, api_controller
from .model import (
    ModelAsyncEndpointFactory,
    ModelConfig,
    ModelControllerBuilder,
    ModelEndpointFactory,
    ModelPagination,
    ModelSchemaConfig,
    ModelService,
    ModelServiceBase,
)
from .response import Detail, Id, Ok
from .route import (
    Route,
    RouteInvalidParameterException,
    http_delete,
    http_generic,
    http_get,
    http_patch,
    http_post,
    http_put,
    route,
)
from .route.route_functions import AsyncRouteFunction, RouteFunction

__all__ = [
    "api_controller",
    "route",
    "http_get",
    "http_post",
    "http_put",
    "http_delete",
    "http_patch",
    "http_generic",
    "ControllerBase",
    "RouteInvalidParameterException",
    "AsyncRouteFunction",
    "RouteFunction",
    "Route",
    "Ok",
    "Id",
    "Detail",
    "ModelControllerBase",
    "ModelConfig",
    "ModelService",
    "ModelSchemaConfig",
    "ModelControllerBuilder",
    "ModelPagination",
    "ModelServiceBase",
    "ModelEndpointFactory",
    "ModelAsyncEndpointFactory",
]


def __getattr__(name: str) -> t.Any:
    if name == "RouteContext":
        warnings.warn(
            "RouteContext is deprecated and will be removed in a future version.",
            DeprecationWarning,
            stacklevel=2,
        )
        from ninja_extra.context import RouteContext

        return RouteContext
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")



from typing import Any, Dict, List, Optional

from django.conf import settings as django_settings
from django.core.signals import setting_changed
from ninja.pagination import PaginationBase
from ninja.throttling import BaseThrottle
from pydantic import BaseModel, Field
from typing_extensions import Annotated

from ninja_extra.conf.decorator import AllowTypeOfSource
from ninja_extra.interfaces.ordering import OrderingBase
from ninja_extra.interfaces.route_context import RouteContextBase
from ninja_extra.interfaces.searching import SearchingBase
from ninja_extra.lazy import LazyStrImport

_GenericModelValidator = AllowTypeOfSource(
    validator=lambda source, value: (
        isinstance(value, LazyStrImport)
        or isinstance(value, type)
        and issubclass(value, source)
    ),
    error_message=lambda source,
    value: f"Expected type of {source.__name__}, received: {type(value)}",
)

PaginationClassHandlerType = Annotated[PaginationBase, _GenericModelValidator]
ThrottlingClassHandlerType = Annotated[BaseThrottle, _GenericModelValidator]
SearchingClassHandlerType = Annotated[SearchingBase, _GenericModelValidator]
OrderingClassHandlerType = Annotated[OrderingBase, _GenericModelValidator]
RouteContextHandlerType = Annotated[RouteContextBase, _GenericModelValidator]

_InjectorModuleValidator = AllowTypeOfSource(
    validator=lambda source, value: value is not None,
    error_message=lambda source,
    value: f"Expected PaginationBase object, received: {type(value)}",
)
InjectorModuleHandlerType = Annotated[Any, _InjectorModuleValidator]


class UserDefinedSettingsMapper:
    def __init__(self, data: dict) -> None:
        self.__dict__ = dict(NinjaEXTRA_SETTINGS_DEFAULTS, **data)


NinjaEXTRA_SETTINGS_DEFAULTS = {
    "INJECTOR_MODULES": [],
    "PAGINATION_CLASS": "ninja.pagination.LimitOffsetPagination",
    "THROTTLE_CLASSES": [
        "ninja_extra.throttling.AnonRateThrottle",
        "ninja_extra.throttling.UserRateThrottle",
    ],
    "THROTTLE_RATES": {"user": None, "anon": None},
    "ORDERING_CLASS": "ninja_extra.ordering.Ordering",
    "SEARCHING_CLASS": "ninja_extra.searching.Searching",
    "ROUTE_CONTEXT_CLASS": "ninja_extra.context.RouteContext",
}

USER_SETTINGS = UserDefinedSettingsMapper(
    getattr(django_settings, "NINJA_EXTRA", NinjaEXTRA_SETTINGS_DEFAULTS)
)


class NinjaExtraSettings(BaseModel):
    class Config:
        from_attributes = True
        validate_assignment = True

    PAGINATION_CLASS: PaginationClassHandlerType = Field(  # type: ignore[assignment]
        "ninja.pagination.LimitOffsetPagination",
    )
    PAGINATION_PER_PAGE: int = Field(100)
    THROTTLE_RATES: Dict[str, Optional[str]] = Field(
        {"user": "1000/day", "anon": "100/day"}
    )
    THROTTLE_CLASSES: List[ThrottlingClassHandlerType] = Field(
        [
            "ninja_extra.throttling.AnonRateThrottle",  # type: ignore[list-item]
            "ninja_extra.throttling.UserRateThrottle",  # type: ignore[list-item]
        ]
    )
    NUM_PROXIES: Optional[int] = None
    INJECTOR_MODULES: List[InjectorModuleHandlerType] = []
    ORDERING_CLASS: OrderingClassHandlerType = Field(  # type: ignore[assignment]
        "ninja_extra.ordering.Ordering",
    )
    SEARCHING_CLASS: SearchingClassHandlerType = Field(  # type: ignore[assignment]
        "ninja_extra.searching.Searching",
    )
    ROUTE_CONTEXT_CLASS: RouteContextHandlerType = Field(  # type: ignore[assignment]
        "ninja_extra.context.RouteContext",
    )


settings = NinjaExtraSettings.model_validate(USER_SETTINGS)


def reload_settings(*args: Any, **kwargs: Any) -> None:  # pragma: no cover
    global settings

    setting, value = kwargs["setting"], kwargs["value"]

    if setting == "NINJA_EXTRA":
        settings = NinjaExtraSettings.model_validate(UserDefinedSettingsMapper(value))


setting_changed.connect(reload_settings)  # pragma: no cover



import inspect
import typing as t

from ninja.constants import NOT_SET
from ninja.signature import is_async

from ninja_extra.interfaces.searching import SearchingBase
from ninja_extra.lazy import settings_lazy

from .operations import (
    AsyncSearcheratorOperation,
    SearcheratorOperation,
)


@t.overload
def searching() -> t.Callable[..., t.Any]:  # pragma: no cover
    ...


@t.overload
def searching(
    func_or_searching_class: t.Any = NOT_SET, **searching_params: t.Any
) -> t.Callable[..., t.Any]:  # pragma: no cover
    ...


def searching(
    func_or_searching_class: t.Any = NOT_SET, **searching_params: t.Any
) -> t.Callable[..., t.Any]:
    isfunction = inspect.isfunction(func_or_searching_class)
    isnotset = func_or_searching_class == NOT_SET

    searching_class: t.Type[SearchingBase] = t.cast(
        t.Type[SearchingBase], settings_lazy().SEARCHING_CLASS
    )

    if isfunction:
        return _inject_searcherator(func_or_searching_class, searching_class)

    if not isnotset:
        searching_class = func_or_searching_class

    def wrapper(func: t.Callable[..., t.Any]) -> t.Any:
        return _inject_searcherator(func, searching_class, **searching_params)

    return wrapper


def _inject_searcherator(
    func: t.Callable[..., t.Any],
    searching_class: t.Type[SearchingBase],
    **searching_params: t.Any,
) -> t.Callable[..., t.Any]:
    searcherator: SearchingBase = searching_class(**searching_params)
    searcherator_kwargs_name = "searching"
    searcherator_operation_class = SearcheratorOperation
    if is_async(func):
        searcherator_operation_class = AsyncSearcheratorOperation
    searcherator_operation = searcherator_operation_class(
        searcherator=searcherator,
        view_func=func,
        searcherator_kwargs_name=searcherator_kwargs_name,
    )

    return searcherator_operation.as_view



import typing as t

from django.utils.module_loading import import_string
from pydantic import GetCoreSchemaHandler, GetJsonSchemaHandler
from pydantic.json_schema import JsonSchemaValue
from pydantic_core.core_schema import CoreSchema, with_info_plain_validator_function

from ninja_extra.lazy import LazyStrImport
from ninja_extra.shortcuts import fail_silently


class AllowTypeOfSource:
    def __init__(
        self,
        schema: t.Optional[t.Dict[str, t.Any]] = None,
        validator: t.Optional[t.Callable[..., bool]] = None,
        error_message: t.Optional[t.Callable[..., str]] = None,
    ) -> None:
        self._schema = schema
        self.validator = validator
        self.error_message = error_message

    def __get_pydantic_core_schema__(
        self,
        source: t.Type[t.Any],
        handler: GetCoreSchemaHandler,
    ) -> CoreSchema:
        def validate(value: t.Any, *args: t.Any) -> t.Any:
            if isinstance(value, str):
                try_import_value = fail_silently(import_string, value)
                if try_import_value is not None:
                    value = try_import_value
                else:
                    value = LazyStrImport(value)

            if (self.validator and not self.validator(source, value)) or (
                not self.validator and not isinstance(value, source)
            ):
                self._handle_error(source, value)
            return value

        return with_info_plain_validator_function(validate)

    def _handle_error(self, source: t.Any, value: t.Any) -> None:
        error_message = (
            f"Expected an instance of {source}, got an instance of {type(value)}"
            if self.error_message is None
            else self.error_message(source, value)
        )
        raise ValueError(error_message)

    def __get_pydantic_json_schema__(
        self, core_schema: CoreSchema, handler: GetJsonSchemaHandler
    ) -> JsonSchemaValue:  # pragma: no cover
        return self._schema  # type:ignore[return-value]



import logging
import typing as t
from functools import wraps

from asgiref.sync import sync_to_async
from django.http import HttpRequest

from ninja_extra.interfaces.ordering import OrderingBase
from ninja_extra.shortcuts import add_ninja_contribute_args

logger = logging.getLogger()

if t.TYPE_CHECKING:  # pragma: no cover
    from ninja_extra.controllers.base import ControllerBase


class OrderatorOperation:
    def __init__(
        self,
        *,
        orderator: OrderingBase,
        view_func: t.Callable,
        orderator_kwargs_name: str = "ordering",
    ) -> None:
        self.orderator = orderator
        self.orderator_kwargs_name = orderator_kwargs_name
        self.view_func = view_func

        orderator_view = self.get_view_function()
        self.as_view = wraps(view_func)(orderator_view)
        add_ninja_contribute_args(
            self.as_view,
            (
                self.orderator_kwargs_name,
                self.orderator.Input,
                self.orderator.InputSource,
            ),
        )
        orderator_view.orderator_operation = self  # type:ignore[attr-defined]

    @property
    def view_func_has_kwargs(self) -> bool:  # pragma: no cover
        return self.orderator.pass_parameter is not None

    def get_view_function(self) -> t.Callable:
        def as_view(
            request_or_controller: t.Union["ControllerBase", HttpRequest],
            *args: t.Any,
            **kw: t.Any,
        ) -> t.Any:
            func_kwargs = dict(**kw)
            ordering_params = func_kwargs.pop(self.orderator_kwargs_name)
            if self.orderator.pass_parameter:
                func_kwargs[self.orderator.pass_parameter] = ordering_params

            items = self.view_func(request_or_controller, *args, **func_kwargs)
            if (
                isinstance(items, tuple)
                and len(items) == 2
                and isinstance(items[0], int)
            ):
                return items
            return self.orderator.ordering_queryset(items, ordering_params)

        return as_view


class AsyncOrderatorOperation(OrderatorOperation):
    def get_view_function(self) -> t.Callable:
        async def as_view(
            request_or_controller: t.Union["ControllerBase", HttpRequest],
            *args: t.Any,
            **kw: t.Any,
        ) -> t.Any:
            func_kwargs = dict(**kw)
            ordering_params = func_kwargs.pop(self.orderator_kwargs_name)
            if self.orderator.pass_parameter:
                func_kwargs[self.orderator.pass_parameter] = ordering_params

            items = await self.view_func(request_or_controller, *args, **func_kwargs)

            if (
                isinstance(items, tuple)
                and len(items) == 2
                and isinstance(items[0], int)
            ):
                return items

            ordering_queryset = t.cast(
                t.Callable, sync_to_async(self.orderator.ordering_queryset)
            )
            return await ordering_queryset(items, ordering_params)

        return as_view



import typing as t
from functools import wraps

from asgiref.sync import sync_to_async

from ninja_extra.interfaces.searching import SearchingBase
from ninja_extra.shortcuts import add_ninja_contribute_args

if t.TYPE_CHECKING:  # pragma: no cover
    from ninja_extra.controllers import ControllerBase


class SearcheratorOperation:
    def __init__(
        self,
        *,
        searcherator: SearchingBase,
        view_func: t.Callable,
        searcherator_kwargs_name: str = "searching",
    ) -> None:
        self.searcherator = searcherator
        self.searcherator_kwargs_name = searcherator_kwargs_name
        self.view_func = view_func

        searcherator_view = self.get_view_function()
        self.as_view = wraps(view_func)(searcherator_view)
        add_ninja_contribute_args(
            self.as_view,
            (
                self.searcherator_kwargs_name,
                self.searcherator.Input,
                self.searcherator.InputSource,
            ),
        )
        searcherator_view.searcherator_operation = self  # type:ignore[attr-defined]

    @property
    def view_func_has_kwargs(self) -> bool:  # pragma: no cover
        return self.searcherator.pass_parameter is not None

    def get_view_function(self) -> t.Callable:
        def as_view(controller: "ControllerBase", *args: t.Any, **kw: t.Any) -> t.Any:
            func_kwargs = dict(**kw)
            searching_params = func_kwargs.pop(self.searcherator_kwargs_name)
            if self.searcherator.pass_parameter:
                func_kwargs[self.searcherator.pass_parameter] = searching_params

            items = self.view_func(controller, *args, **func_kwargs)
            if (
                isinstance(items, tuple)
                and len(items) == 2
                and isinstance(items[0], int)
            ):
                return items

            return self.searcherator.searching_queryset(items, searching_params)

        return as_view


class AsyncSearcheratorOperation(SearcheratorOperation):
    def get_view_function(self) -> t.Callable:
        async def as_view(
            controller: "ControllerBase", *args: t.Any, **kw: t.Any
        ) -> t.Any:
            func_kwargs = dict(**kw)
            searching_params = func_kwargs.pop(self.searcherator_kwargs_name)
            if self.searcherator.pass_parameter:
                func_kwargs[self.searcherator.pass_parameter] = searching_params

            items = await self.view_func(controller, *args, **func_kwargs)

            if (
                isinstance(items, tuple)
                and len(items) == 2
                and isinstance(items[0], int)
            ):
                return items

            searching_queryset = t.cast(
                t.Callable, sync_to_async(self.searcherator.searching_queryset)
            )
            return await searching_queryset(items, searching_params)

        return as_view



import inspect
import logging
import typing as t

from ninja.constants import NOT_SET
from ninja.signature import is_async

from ninja_extra.interfaces.ordering import OrderingBase
from ninja_extra.lazy import settings_lazy

from .operation import AsyncOrderatorOperation, OrderatorOperation

logger = logging.getLogger()


@t.overload
def ordering() -> t.Callable[..., t.Any]:  # pragma: no cover
    ...


@t.overload
def ordering(
    func_or_ordering_class: t.Any = NOT_SET, **paginator_params: t.Any
) -> t.Callable[..., t.Any]:  # pragma: no cover
    ...


def ordering(
    func_or_ordering_class: t.Any = NOT_SET, **ordering_params: t.Any
) -> t.Callable[..., t.Any]:
    isfunction = inspect.isfunction(func_or_ordering_class)
    isnotset = func_or_ordering_class == NOT_SET

    ordering_class: t.Type[OrderingBase] = t.cast(
        t.Type[OrderingBase], settings_lazy().ORDERING_CLASS
    )

    if isfunction:
        return _inject_orderator(func_or_ordering_class, ordering_class)

    if not isnotset:
        ordering_class = func_or_ordering_class

    def wrapper(func: t.Callable[..., t.Any]) -> t.Any:
        return _inject_orderator(func, ordering_class, **ordering_params)

    return wrapper


def _inject_orderator(
    func: t.Callable[..., t.Any],
    ordering_class: t.Type[OrderingBase],
    **ordering_params: t.Any,
) -> t.Callable[..., t.Any]:
    orderator: OrderingBase = ordering_class(**ordering_params)
    orderator_kwargs_name = "ordering"
    orderator_operation_class = OrderatorOperation
    if is_async(func):
        orderator_operation_class = AsyncOrderatorOperation
    orderator_operation = orderator_operation_class(
        orderator=orderator, view_func=func, orderator_kwargs_name=orderator_kwargs_name
    )

    return orderator_operation.as_view
